/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Retrieving Resource Items
    Resource Items are retrieved just as DDOs are.  Since XDOs are are DDOs with extended / 
    extra funcitonality, Sample SItemRetrievalICM may be used for all non-Resource related
    functionality.
    
    Resource Content Access: File, Stream, & Memory
        As shown in SResourceItemCreationICM, Resource Content may be stored directly 
        from a file, from a stream, or loaded into memory and later saved.  The 
        method in which the Resource Content was saved DOES NOT affect how it may
        be retrieved.  The same options are available when accessing the content,
        regardless of how it was originally stored.  A Resource Item added using the 
        "Direct from File" may be accessed later as a Stream or by Local Memory.
    
    Maximizing Efficiency
        Resource Item may be much larger than regular Items, which means that if the 
        Resource Content may not be needed, retrieval of the Item without retrieving the 
        Resource Content is a good solution for faster retrievals overall.  The Resource 
        Content may be retrieved if and when it is specifically needed.  An example of 
        such usage would be in an application displaying a list of hundreds of documents.
        Populating the list does not require resource content.  Only when the user chooses
        to view a particular document, it must then be retrieved.
    
    Retrieval Options
        When actually retrieving the DDO/XDO from the datastore, retrieval options may be 
        specified.  The full set of retrieval options are explained in SItemRetrievalICM.
        The subset relating specifically to Resource Content is shown below.

        Retrieval Option      Constant                     Explanation
        --------------------  ---------------------------  -------------------------------------
        No Resource Content   DK_CM_CONTENT_NO             If it is a Resource Item, do not 
                                                           retrieve the Resource object content.
        ........................................................................................
        Resource Content      DK_CM_CONTENT_YES            If it is a Resource Item, retrieve 
                                                           the Resource object content.
        ........................................................................................
        Only Resource Content DK_CM_CONTENT_ONLY           If it is a Resource Item, retrieve 
                                                           only the Resource object content.
        ----------------------------------------------------------------------------------------
        Note: Constants shown above are defined in DKConstant.h

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKConstantICM.h>
#include <DKDatastoreICM.hpp>
#include <DKLobICM.hpp>
#include <DKTextICM.hpp>
#include <DKImageICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SResourceItemRetrievalICM.hpp"

/************************************************************************************************
 *          FILENAME: SResourceItemRetrievalICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Retrieving Resource items, Accessing Resource Content, Maximizing
 *                    Efficiency, Resource-Related Retrieval Options, Validating Files & 
 *                    Streams.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Retrieving Resource Items
 *                    Resource Content Access: File, Stream, & Memory
 *                    Maximizing Efficiency
 *                    Resource-Related Retrieval Options
 *                    Accessing Resource Content
 *                    Accessing As File
 *                    Accessing As Stream
 *                    Validating Files
 *                    Validating Streams
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SResourceItemRetrievalICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                         - SAttributeDefinitionCreationICM
 *                         - SAttributeGroupDefCreationICM
 *                         - SReferenceAttrDefCreationICM
 *                         - SItemTypeCreationICM
 *                    The Item Types must be set to use the specified default Resource Manager.
 *                    Plese run the following samples.
 *                         - SResourceMgrDefSetDefaultICM
 *                         - SSMSCollectionDefSetDefaultICM
 *                    NOTE:  Sample SSampleModelBuildICM pulls together all of the above.  It 
 *                           must be edited first for the RM Confiuration information.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SResourceItemRetrievalICM_launch.cpp
 *                    SResourceItemRetrievalICM.hpp
 *                    SResourceItemICM_Document1.doc
 *                    SResourceItemICM_Document2.doc
 *                    SResourceItemICM_Text1.txt
 *                    SResourceItemICM_Text2.txt
 *                    SResourceItemICM_Image1.bmp
 *                    SResourceItemICM_Image2.bmp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SResourceItemRetrievalICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                        << endl;
            cout << "  SResourceItemRetrievalICM <database> <userName> <password>"   << endl;
            cout << "  *** Some parameters not specified, using defaults..."         << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SResourceItemRetrievalICM"      << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Database: " << database                         << endl;
        cout << " UserName: " << userName                         << endl;
        cout << "===========================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // Create Resource Objects for Retrieval Demonstration
            //-------------------------------------------------------------
            // In order to demonstrate retrieval of Resource Items, a few
            // Resource Items must first be created.
            cout << "Creating Resource Objects..." << endl;

                DKLobICM*     lob = (DKLobICM*)   dsICM->createDDO("S_lob",   DK_CM_DOCUMENT); // May be any semantic type.
                DKTextICM*   text = (DKTextICM*)  dsICM->createDDO("S_text",  DK_CM_ITEM);     // Resource Type determined by XDO Classification
                DKImageICM* image = (DKImageICM*) dsICM->createDDO("S_image", DK_CM_ITEM);     // in the Item Type Definition.

                // Set Mime Type / Type of Content
                lob->setMimeType("application/msword");
                text->setMimeType("text/plain");
                image->setMimeType("image/bmp");
                
                // Store Resource Content & Save Resource Item
                lob->add("SResourceItemICM_Document1.doc");  // Add it to the datastore,
                text->add("SResourceItemICM_Text1.txt");     // storing the content from
                image->add("SResourceItemICM_Picture1.bmp"); // the specified file.

            cout << "Created Resource Objects." << endl;

            //-------------------------------------------------------------
            // Recreating XDO objects given PIDs.
            //-------------------------------------------------------------
            // In order to retrieve an Item either the PID Information must
            // be known or Query / Search must be used.  This sample will 
            // use the PID Strings of the items added above.
            cout << "Recreating XDO Objects Given PIDs..." << endl;
            
                // First we will need to know what items to retrieve.  The PID information is needed.
                DKString lobPidString   = ((DKPidICM*)  lob->getPidObject())->pidString();
                DKString textPidString  = ((DKPidICM*) text->getPidObject())->pidString();
                DKString imagePidString = ((DKPidICM*)image->getPidObject())->pidString();

                delete(lob);            // Free Memory since objects not needed anymore.
                delete(text);
                delete(image);

                // Knowing Pid strings, let's create new, blank DDOs.
                lob   = (DKLobICM*)   dsICM->createDDO(lobPidString);
                text  = (DKTextICM*)  dsICM->createDDO(textPidString);
                image = (DKImageICM*) dsICM->createDDO(imagePidString);

            cout << "Recreated XDO Objects Given PIDs." << endl;

            //-------------------------------------------------------------
            // Retrieving Resource Item, Saving Content Directly to File.
            //-------------------------------------------------------------
            // Regardless of how the Content was originally stored, it may 
            // be written directly to a file when retrieved.
            cout << "Retrieving Resource ITem, Saving Content Directly to File..." << endl;

                lob->retrieve("myDocument.doc");     // Item will be retrieved and
                text->retrieve("myText.txt");        // resource content will be
                image->retrieve("myPicture.bmp");    // saved in the specified file.

            cout << "Retrieved Resource ITem, Saved Content Directly to File." << endl;

            //-------------------------------------------------------------
            // Quick Validation Check Prooving Identical to Original
            //-------------------------------------------------------------
            // Using funcitons provided at the bottom, we will perform a quick
            // validation check to validate and prove that the new files
            // are the exact original specified.
            cout << "Validing..." << endl;
            
                if(!compareFiles("SResourceItemICM_Document1.doc","myDocument.doc"))
                    throw DKException("Retrieved Document not identical to original.");
                if(!compareFiles("SResourceItemICM_Text1.txt","myText.txt"))
                    throw DKException("Retrieved Text File not identical to original.");
                if(!compareFiles("SResourceItemICM_Picture1.bmp","myPicture.bmp"))
                    throw DKException("Retrieved Picture not identical to original.");

            cout << "Valided." << endl;

            //-------------------------------------------------------------
            // Retrieving Resource, Storing Content to Local Memory
            //-------------------------------------------------------------
            // One of the most commonly used methods holds the Resource Content
            // in local memory.  The option below stores it in local memory,
            // alowing easy access to it at a later time.
            cout << "Retrieving Resource Item, Storing Content in Local Memory..." << endl;

                lob->retrieve(DK_CM_CONTENT_YES);     // Retrieve content
                text->retrieve(DK_CM_CONTENT_YES);    // with object.
                image->retrieve(DK_CM_CONTENT_YES);
                
                // NOTE:  If the content does not need to be retrieved at this point, as
                //        explained in the header documentation, the following can be used:
                // lob.retrieve(DKConstant.DK_CM_CONTENT_NO);

            cout << "Retrieved Resource Item, Stored Content in Local Memory." << endl;

            //-------------------------------------------------------------
            // Store content from resource memory to a file.
            //-------------------------------------------------------------
            // Now that the Resource Item has been retrieved, with content in
            // memory, the content may be easily stored to a file.
            cout << "Storing Content from Resource Memory to a File..." << endl;
    
                lob->getContentToClientFile("myDocument.doc",1);  // "1" means overwrite any existing file.
                text->getContentToClientFile("myText.txt",1);     // Please refer to the Applicationg Programming Reference
                image->getContentToClientFile("myPicture.bmp",1); // (Javadoc) for more information.

            cout << "Stored Content from Resource Memory to a File." << endl;

            //-------------------------------------------------------------
            // Make content available through a stream object
            //-------------------------------------------------------------
            cout << "Making Content Available Through a Stream Object..." << endl;

                istream* inputStream1 = lob->getInputStream(-1,-1,-1); // -1 is the default.
                istream* inputStream2 = text->getInputStream(-1,-1,-1); // -1 is the default.
                istream* inputStream3 = image->getInputStream(-1,-1,-1); // -1 is the default.

            cout << "Made Content Available Through a Stream Object." << endl;

            //-------------------------------------------------------------
            // Quick Validation Check Prooving Identical to Original
            //-------------------------------------------------------------
            cout << "Validing Streams..." << endl;

                if(!compareStreamToFile(inputStream1,"SResourceItemICM_Document1.doc"))
                    throw DKException("Retrieved Document Stream not identical to original.");
                if(!compareStreamToFile(inputStream2,"SResourceItemICM_Text1.txt"))
                    throw DKException("Retrieved Text Steam not identical to original.");
                if(!compareStreamToFile(inputStream3,"SResourceItemICM_Picture1.bmp"))
                    throw DKException("Retrieved Picture Stream not identical to original.");

            cout << "Valided Streams." << endl;

            delete inputStream1;
            delete inputStream2;
            delete inputStream3;

            //-------------------------------------------------------------
            // Quick Validation for 2nd Set of Files Retrieved.
            //-------------------------------------------------------------
            cout << "Validing..." << endl;
            
                if(!compareFiles("SResourceItemICM_Document1.doc","myDocument.doc"))
                    throw DKException("Retrieved Document not identical to original.");
                if(!compareFiles("SResourceItemICM_Text1.txt","myText.txt"))
                    throw DKException("Retrieved Text File not identical to original.");
                if(!compareFiles("SResourceItemICM_Picture1.bmp","myPicture.bmp"))
                    throw DKException("Retrieved Picture not identical to original.");

                delete(lob);          // Free Memory
                delete(text);
                delete(image);

            cout << "Valided." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Complare two files, return true if same, false if not.  Performs Binary
    * Comparison.
    * @param file1  One file to compare.
    * @param file2  A second file to compare to the first.
    * @return  Returns true if they are the same, false if not.
    */
    bool SResourceItemRetrievalICM::compareFiles(char* file1, char* file2){
        ifstream* f1 = new ifstream(file1,ios::binary);
        if(!f1) throw DKException(DKString("Cannot open input file1 '")+DKString(file1)+DKString("' in SResourceItemRetrievalICM::compareFiles(file1,file2)."));
        ifstream* f2 = new ifstream(file2,ios::binary);
        if(!f2) throw DKException(DKString("Cannot open input file2 '")+DKString(file2)+DKString("' in SResourceItemRetrievalICM::compareFiles(file1,file2)."));
        bool areEqual = true;
        bool stop = false;
        // read a byte from each until both hit end.
        while(!stop){ // keep going until told to stop.
            int byte1 = f1->get();
            int byte2 = f2->get();
            if( (f1->eof()) && (f2->eof()) ){ // if both reached the end, stop now
                areEqual = true;
                stop = true;
            }
            else if( (f1->eof()) && (!f2->eof()) ){ // if one ends before the other, they are not the same.
                areEqual = false;
                stop = true;
            }
            else if( (!f1->eof()) && (f2->eof()) ){ // if one ends before the other, they are not the same.
                areEqual = false;
                stop = true;
            }
            if(byte1 != byte2){ // if at any point, they differ, return false.
                areEqual = false;
                stop = true;
            }
        }
        f1->close(); // close the file
        f2->close(); // close the file

        delete(f1);  // Free Memory
        delete(f2);
        
        return(areEqual);
    }
   /**
    * Complare a file and an input stream, return true if same, false if not.  Performs
    * Binary Comparison.
    * @param inputStream  InputStream to compare.
    * @param file  A file to compare to the input stream.
    * @return  Returns true if they are the same, false if not.
    */
    bool SResourceItemRetrievalICM::compareStreamToFile(istream* is, char* file){
        ifstream* f = new ifstream(file,ios::binary);
        if(!f) throw DKException(DKString("Cannot open input file '")+DKString(file)+DKString("' in SResourceItemRetrievalICM::compareFiles(istream,file)."));
        bool areEqual = true;
        bool stop = false;
        // read a byte from each until both hit end.
        while(!stop){ // keep going until told to stop.
            int byte1 = is->get();
            int byte2 = f->get();
            if( (is->eof()) && (f->eof()) ){ // if both reached the end, stop now
                areEqual = true;
                stop = true;
            }
            else if( (is->eof()) && (!f->eof()) ){ // if one ends before the other, they are not the same.
                areEqual = false;
                stop = true;
            }
            else if( (!is->eof()) && (f->eof()) ){ // if one ends before the other, they are not the same.
                areEqual = false;
                stop = true;
            }
            if(byte1 != byte2){ // if at any point, they differ, return false.
                areEqual = false;
                stop = true;
            }
        }
        f->close(); // close the file
        delete(f);
        return(areEqual);
    }

   /**
    * Determine the length in bytes of the specified file.
    * @param file  The file to determine the legth of.
    * @return  Returns the size of the specified file.
    */
    int SResourceItemRetrievalICM::getFileLength(char* file){
        ifstream* f = new ifstream(file,ios::binary);
        if(!f) throw DKException(DKString("Cannot open input file '")+DKString(file)+DKString("' in SResourceItemRetrievalICM::compareFiles(istream,file)."));
        bool stop = false;
        int length = 0;  // Number of bytes in the file.
        // read a byte from hit end.
        while(!stop){ // keep going until told to stop.
            int byte = f->get();
            if(f->eof()) // if both reached the end, stop now
                stop = true;
            else
                length++;
        }
        f->close(); // close the file
        delete(f);
        return(length);
    }
